@@ -10,9 +10,10 @@ use crate::install_context::InstallContext;
 use crate::toolset::{ToolVersion, ToolVersionOptions, Toolset, ToolsetBuilder};
 use crate::ui::multi_progress_report::MultiProgressReport;
 use crate::ui::progress_report::SingleReport;
-use eyre::Result;
+use eyre::{Result, eyre};
 use indexmap::IndexMap;
 use itertools::Itertools;
+use regex::Regex;
 use std::fmt::Debug;
 use std::str::FromStr;
 use versions::Versioning;
@@ -48,15 +49,40 @@ impl Backend for PIPXBackend {
     fn _list_remote_versions(&self) -> eyre::Result<Vec<String>> {
         match self.tool_name().parse()? {
             PipxRequest::Pypi(package) => {
-                let url = format!("https://pypi.org/pypi/{}/json", package);
-                let data: PypiPackage = HTTP_FETCH.json(url)?;
-                let versions = data
-                    .releases
-                    .keys()
-                    .map(|v| v.to_string())
-                    .sorted_by_cached_key(|v| Versioning::new(v))
-                    .collect();
-                Ok(versions)
+                let registry_url = self.get_registry_url()?;
+                if registry_url.contains("/json") {
+                    debug!("Fetching JSON for {}", package);
+                    let url = format!("https://pypi.org/pypi/{}/json", package);
+                    let data: PypiPackage = HTTP_FETCH.json(url)?;
+                    let versions = data
+                        .releases
+                        .keys()
+                        .map(|v| v.to_string())
+                        .sorted_by_cached_key(|v| Versioning::new(v))
+                        .collect();
+
+                    Ok(versions)
+                } else {
+                    debug!("Fetching HTML for {}", package);
+                    let url = format!("https://pypi.org/simple/{}/", package);
+                    let html = HTTP_FETCH.get_html(url)?;
+
+                    let version_re = regex!(r#"href=["'].*?/([^/]+)\.tar\.gz["']"#);
+                    let versions: Vec<String> = version_re
+                        .captures_iter(&html)
+                        .filter_map(|cap| {
+                            let filename = cap.get(1)?.as_str();
+                            let escaped_package = regex::escape(&package);
+                            let re_str = format!("^{}-(.+)$", escaped_package);
+                            let pkg_re = regex::Regex::new(&re_str).ok()?;
+                            let pkg_version = pkg_re.captures(filename)?.get(1)?.as_str();
+                            Some(pkg_version.to_string())
+                        })
+                        .sorted_by_cached_key(|v| Versioning::new(v))
+                        .collect();
+
+                    Ok(versions)
+                }
             }
             PipxRequest::Git(url) if url.starts_with("https://github.com/") => {
                 let repo = url.strip_prefix("https://github.com/").unwrap();
@@ -71,9 +97,39 @@ impl Backend for PIPXBackend {
         self.latest_version_cache
             .get_or_try_init(|| match self.tool_name().parse()? {
                 PipxRequest::Pypi(package) => {
-                    let url = format!("https://pypi.org/pypi/{}/json", package);
-                    let pkg: PypiPackage = HTTP_FETCH.json(url)?;
-                    Ok(Some(pkg.info.version))
+                    let registry_url = self.get_registry_url()?;
+                    if registry_url.contains("/json") {
+                        debug!("Fetching JSON for {}", package);
+                        let url = format!("https://pypi.org/pypi/{}/json", package);
+                        let pkg: PypiPackage = HTTP_FETCH.json(url)?;
+                        Ok(Some(pkg.info.version))
+                    } else {
+                        debug!("Fetching HTML for {}", package);
+                        let url = format!("https://pypi.org/simple/{}/", package);
+                        let html = HTTP_FETCH.get_html(url)?;
+
+                        let version_re = regex!(r#"href=["'].*?/([^/]+)\.tar\.gz["']"#);
+                        let version = version_re
+                            .captures_iter(&html)
+                            .filter_map(|cap| {
+                                let filename = cap.get(1)?.as_str();
+                                let escaped_package = regex::escape(&package);
+                                let re_str = format!("^{}-(.+)$", escaped_package);
+                                let pkg_re = regex::Regex::new(&re_str).ok()?;
+                                let pkg_version = pkg_re.captures(filename)?.get(1)?.as_str();
+                                Some(pkg_version.to_string())
+                            })
+                            .filter(|v| {
+                                !v.contains("dev")
+                                    && !v.contains("a")
+                                    && !v.contains("b")
+                                    && !v.contains("rc")
+                            })
+                            .sorted_by_cached_key(|v| Versioning::new(v))
+                            .next_back();
+
+                        Ok(version)
+                    }
                 }
                 _ => self.latest_version(Some("latest".into())),
             })
@@ -136,6 +192,22 @@ impl PIPXBackend {
         }
     }
 
+    fn get_registry_url(&self) -> eyre::Result<String> {
+        let registry_url = SETTINGS.pipx.registry_url.clone();
+
+        debug!("Pipx registry URL: {}", registry_url);
+
+        let re = Regex::new(r"^(http|https)://.*\{\}.*$").unwrap();
+
+        if !re.is_match(&registry_url) {
+            return Err(eyre!(
+                "Registry URL must be a valid URL and contain a {{}} placeholder"
+            ));
+        }
+
+        Ok(registry_url)
+    }
+
     pub fn reinstall_all() -> Result<()> {
         let config = Config::load()?;
         let ts = ToolsetBuilder::new().build(&config)?;
@@ -245,20 +317,6 @@ impl PipxRequest {
     }
 }
 
-impl FromStr for PipxRequest {
-    type Err = eyre::Error;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        if let Some(cap) = regex!(r"(git\+)(.*)(\.git)").captures(s) {
-            Ok(PipxRequest::Git(cap.get(2).unwrap().as_str().to_string()))
-        } else if s.contains('/') {
-            Ok(PipxRequest::Git(format!("https://github.com/{s}")))
-        } else {
-            Ok(PipxRequest::Pypi(s.to_string()))
-        }
-    }
-}
-
 #[derive(serde::Deserialize)]
 struct PypiPackage {
     releases: IndexMap<String, Vec<PypiRelease>>,
@@ -272,3 +330,17 @@ struct PypiInfo {
 
 #[derive(serde::Deserialize)]
 struct PypiRelease {}
+
+impl FromStr for PipxRequest {
+    type Err = eyre::Error;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if let Some(cap) = regex!(r"(git\+)(.*)(\.git)").captures(s) {
+            Ok(PipxRequest::Git(cap.get(2).unwrap().as_str().to_string()))
+        } else if s.contains('/') {
+            Ok(PipxRequest::Git(format!("https://github.com/{s}")))
+        } else {
+            Ok(PipxRequest::Pypi(s.to_string()))
+        }
+    }
+}
